// ContentItem.java - leaf content in a NewsML package.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: ContentItem.java,v 1.11 2001/05/23 01:56:17 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import org.w3c.dom.NodeList;


/**
 * News content for presentation to humans.
 *
 * <p>A content item represents a leaf node in the NewsML content tree:
 * it either contains or points to actual content, such as a textual
 * news story, a photo, or a video.  If the content item contains the
 * content, the content will be accessible through the
 * {@link #getDataContent} method; otherwise, the content's URL
 * will be available through the {@link #getHref} method.</p>
 *
 * <p>The {@link #getMediaType}, {@link #getFormat}, and {@link
 * #getNotation} methods all provide redundant information already
 * available through {@link #getMimeType}, and provide that 
 * information in ways that will be less familiar to users; 
 * these redundant methods have been included only for the sake of 
 * completeness.</p>
 *
 * @author Reuters PLC
 * @version 1.0
 */
public interface ContentItem
    extends BaseNode, CatalogNode, CommentNode, HrefNode, IdNode, PrimaryNode
{

    /**
     * Get the URL of the actual content.
     *
     * If the actual content is not included inline in the NewsML
     * package, this method will return the URL where the content can
     * be found.  This method and the {@link #getDataContent} methods
     * are mutually exclusive: one of the two will always return null.
     *
     * @return The URL of the content, or null if the content is
     * included inline.
     * @see #getDataContent
     */
    public String getHref ();


    /**
     * Get the content's media type.
     *
     * NewsML media type information is redundant, filling the same
     * function as the first part of the MIME type (see {@link
     * #getMimeType}).  Typical media types include Text, Audio,
     * Video, and Image.
     *
     * @return A formal name representing the media type, or
     * null if no media type was specified.
     * @see #getMimeType
     */
    public FormalName getMediaType ();


    /**
     * Get the content's format.
     *
     * NewsML format information is redundant, filling the same
     * function as the second part of the MIME type (see {@link
     * #getMimeType}).  Typical media types include ANPA 1312 and
     * NITF.
     *
     * @return A formal name describing the format, or null if no
     * format was specified.
     * @see #getMimeType
     */
    public FormalName getFormat ();


    /**
     * Return the content's MIME type.
     *
     * While this value is a formal name, and may be defined by any
     * vocabulary according to the NewsML document type, it would be a
     * very poor choice for a provider not to use the proper MIME-type
     * definition in IETF <a
     * href="http://www.ietf.org/rfc/rfc2045.txt">RFC 2045</a>.
     *
     * @return A formal name describing the MIME type, or null
     * if no MIME type was specified.
     * @see #getCharacteristics
     */
    public FormalName getMimeType ();


    /**
     * Get the content's data type notation.
     *
     * NewsML notation information is redundant, filling the same
     * role as the MIME type (see {@link #getMimeType}) but using a
     * less familiar representation.
     *
     * @return A formal name describing the data type notation, or
     * null if no notation was specified.
     * @see #getMimeType
     */
    public FormalName getNotation ();


    /**
     * Get the physical characteristics of the news content.
     *
     * Physical characteristics include the physical content's
     * size in bytes.  Note that information on media and format
     * is provided by the MIME type (see {@link #getMimeType}).
     *
     * @return An object describing the physical characteristics,
     * or null if none was specified.
     * @see #getMimeType
     */
    public Characteristics getCharacteristics ();


    /**
     * Get the number of encoding steps for the content.
     *
     * @return The number of encoding steps.
     */
    public int getEncodingCount ();


    /**
     * Get an encoding step for the content.
     *
     * Content included inline may have one or more encoding steps
     * applied to it, such as compression, encryption, or base-64
     * encoding.  This method returns the steps sequentially,
     * starting with the first one applied.
     *
     * @param index The index of the encoding, zero-based, numbered
     * sequentially in document order.
     * @return A string representing the encoding, or null if none was
     * provided at the specified index.
     */
    public String getEncoding (int index);


    /**
     * Get all Encoding children in an array.
     *
     * @return A (possibly-empty) array of Encoding children.
     */
    public String [] getEncoding ();


    /**
     * Get the data for inline news content.
     *
     * If the news content is included inline, this method will
     * return a DOM {@link org.w3c.dom.NodeList NodeList} for the
     * content.  This method and the {@link #getHref} method are
     * mutually exclusive: one of the two will always return null.
     *
     * @return A DOM node list representing the news content, or
     * null if no content was included inline.
     * @see #getHref
     * @see #getDataText
     */
    public NodeList getDataContent ();


    /**
     * Get the text for inline news content.
     *
     * This method returns only the text part of inline news content,
     * ignoring any markup.  It is useful when the inline content
     * represents base-64-encoded binary files (such as graphics)
     * rather than XML markup.
     *
     * @return A string containing the text of the news content, or
     * null if the content was not provided inline.
     * @see #getDataContent
     * @see #getHref
     */
    public String getDataText ();

}

// end of ContentItem.java
