// NewsMLSession.java - meta-information for a NewsML session.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: NewsMLSession.java,v 1.9 2001/05/23 01:56:18 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;

import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;


/**
 * Meta-information for a NewsML session.
 *
 * <p>This interface provides a node-independent access point for
 * getting and setting information about a NewsML session.</p>
 *
 * @author Reuters PLC
 * @version 1.0
 */
public interface NewsMLSession
{

    /**
     * Get the factory for creating new NewsML documents.
     *
     * @return The current NewsML factory (not null).
     */
    public NewsMLFactory getFactory ();


    /**
     * Get the root node of the current NewsML document.
     *
     * @return An object representing the root node; the caller
     * will have to cast it to the appropriate type.
     */
    public BaseNode getRootNode ();


    /**
     * Look up a reference to a NewsML node.
     *
     * <p>If the reference contains '#', the part following
     * corresponds with a Duid in the target document (otherwise uses
     * the root element); if the part preceding '#' is empty, the
     * target document is the current document.</p>
     *
     * @param reference The node reference, as a string in URI
     * reference format (may include a '#').
     * @param useExternal true to follow references in external URLs,
     * false otherwise.
     * @return The NewsML node, if one could be located, or null
     * otherwise.
     */
    public BaseNode findReference (String reference, boolean useExternal)
	throws IOException;


    /**
     * Get the base URL for the document.
     *
     * @return The base URL.
     * @see #resolveHref(String)
     */
    public String getBaseURL ();


    /**
     * Resolve a reference.
     *
     * <p>This method is useful for retrieving an external resource --
     * text or binary -- referenced from a NewsML package.  See the
     * standard Java {@link URL} class for information on how your
     * application can use the URL returned to load remote resources.
     * The method automatically resolves any relative hrefs against
     * the NewsML document's base URL.</p>
     *
     * @param href A (possibly relative) URL to resolve.
     * @return The href resolved against the current document's base URL.
     * @exception MalformedURLException if the href cannot be resolved
     * into a URL (this can be caught as an {@link IOException}).
     * @see #getBaseURL()
     */
    public URL resolveHref (String href)
	throws MalformedURLException;
    
}

// end of NewsMLSession.java
