// PrimaryNode.java - base interface for the main nodes in a NewsML content tree.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// $Id: PrimaryNode.java,v 1.12 2001/05/23 01:56:18 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit;


/**
 * Base interface for the main nodes in the NewsML content tree.
 *
 * <p>There are five basic nodes types in the NewsML content tree:</p>
 *
 * <ol>
 * <li>{@link NewsML},</li>
 * <li>{@link NewsItem},</li>
 * <li>{@link NewsComponent},</li>
 * <li>{@link NewsItemRef}, and</li>
 * <li>{@link ContentItem}.</li>
 * </ol>
 *
 * <p>The basic grammar of the NewsML content tree is as follows:</p>
 *
 * <pre>
 * NewsML:        NewsItem+
 * NewsItem:      NewsComponent?
 * NewsComponent: ((NewsItem|NewsItemRef)+ | NewsComponent+ | ContentItem+)
 * NewsItemRef:   <em>[reference to another NewsItem]</em>
 * ContentItem:   <em>[actual data inline or by reference]</em>
 * </pre>
 *
 * @author Reuters PLC
 * @version 1.0
 */
public interface PrimaryNode
    extends BaseNode, IdNode
{

    /**
     * Constant: node is the top-level NewsML package.
     *
     * @see #getType
     */
    public final static int NEWSML = 1;


    /**
     * Constant: node is a news item.
     *
     * @see #getType
     */
    public final static int NEWS_ITEM = 2;


    /**
     * Constant: node is a news component.
     *
     * @see #getType
     */
    public final static int NEWS_COMPONENT = 3;


    /**
     * Constant: node is a reference to a news item.
     *
     * @see #getType
     */
    public final static int NEWS_ITEM_REF = 4;


    /**
     * Constant: node is a content item.
     *
     * @see #getType
     */
    public final static int CONTENT_ITEM = 5;


    /**
     * Get the type of the node in the NewsML content tree.
     *
     * <p>Use this method to determine the type of a primary node so
     * that you can invoke the appropriate casting method.  Normally,
     * an application will use this method in a switch statement
     * similar to the following (assuming that the do* methods belong
     * to the application):</p>
     *
     * <pre>
     * switch (node.getType()) {
     * case PrimaryNode.NEWSML:
     *   doNewsML(node.asNewsML());
     *   break;
     * case PrimaryNode.NEWS_ITEM:
     *   doNewsItem(node.asNewsItem());
     *   break;
     * case PrimaryNode.NEWS_COMPONENT:
     *   doNewsComponent(node.asNewsComponent());
     *   break;
     * case PrimaryNode.NEWS_ITEM_REF:
     *   doNewsItemRef(node.asNewsItemRef());
     *   break;
     * case PrimaryNode.CONTENT_ITEM:
     *   doContentItem(node.asContentItem());
     *   break;
     * default:
     *   throw new Error("Internal error: unexpected primary node type.");
     * }
     * </pre>
     *
     * <p>This method is redundant with {@link BaseNode#getXMLName} in
     * the {@link BaseNode} interface, since the XML name can
     * unambiguously distinguish the node type.</p>
     *
     * @return A constant indicating the node's type.
     * @see #NEWSML
     * @see #NEWS_ITEM
     * @see #NEWS_COMPONENT
     * @see #NEWS_ITEM_REF
     * @see #CONTENT_ITEM
     */
    public abstract int getType ();


    /**
     * Get the parent of the node in the NewsML content tree.
     *
     * @return The parent node, or null if this is the root.
     * @see #getChild
     */ 
    public PrimaryNode getParent ();


    /**
     * Return the number of children for this node.
     *
     * @return The number of children.
     */
    public int getChildCount ();


    /**
     * Return a child of the node by position.
     *
     * @param index The index of the child, zero-based, numbered
     * sequentially.
     * @return The node representing the specified child, or null
     * if none is available at the index provided.
     * @see #getParent
     */
    public PrimaryNode getChild (int index);


    /**
     * Get an array of primary child nodes.
     *
     * @return A (possibly-empty) array of primary child nodes.
     */
    public PrimaryNode [] getChild ();


    /**
     * Cast this node to a NewsML node.
     *
     * @return A NewsML node, or null if this node is of the wrong type.
     * @see #getType
     * @see #NEWSML
     */
    public NewsML asNewsML ();


    /**
     * Cast this node to a NewsItem node.
     *
     * @return A NewsItem node, or null if this node is of the wrong type.
     * @see #getType
     * @see #NEWS_ITEM
     */
    public NewsItem asNewsItem ();


    /**
     * Cast this node to a NewsComponent node.
     *
     * @return A NewsComponent node, or null if this node is of the wrong type.
     * @see #getType
     * @see #NEWS_COMPONENT
     */
    public NewsComponent asNewsComponent ();


    /**
     * Cast this node to a NewsItemRef node.
     *
     * @return A NewsItemRef node, or null if this node is of the wrong type.
     * @see #getType
     * @see #NEWS_ITEM_REF
     */
    public NewsItemRef asNewsItemRef ();


    /**
     * Cast this node to a ContentItem node.
     *
     * @return A ContentItem node, or null if this node is of the wrong type.
     * @see #getType
     * @see #CONTENT_ITEM
     */
    public ContentItem asContentItem ();


    /**
     * Return the path of nodes from the root to this node.
     *
     * The current node will be included as the last element of the array.
     *
     * @return An array of nodes from the root to the current node.
     */
    public PrimaryNode [] getNodePath ();

}

// end of PrimaryNode.java
