// DOMBaseNode.java - base functionality for all NewsML objects.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC
// $Id: DOMBaseNode.java,v 1.12 2001/05/23 01:56:18 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import org.w3c.dom.Node;

import org.newsml.toolkit.AssignedFormalName;
import org.newsml.toolkit.AssignedOriginText;
import org.newsml.toolkit.AssignmentNode;
import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.Catalog;
import org.newsml.toolkit.CatalogNode;
import org.newsml.toolkit.Comment;
import org.newsml.toolkit.CommentNode;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.HrefNode;
import org.newsml.toolkit.IdNode;
import org.newsml.toolkit.LanguageNode;
import org.newsml.toolkit.NewsMLSession;
import org.newsml.toolkit.OriginText;
import org.newsml.toolkit.PartyList;
import org.newsml.toolkit.Property;
import org.newsml.toolkit.PropertyNode;
import org.newsml.toolkit.RevisionHistory;
import org.newsml.toolkit.Text;


/**
 * Base class for all NewsML objects.
 *
 * <p>This base class provides access to the underlying DOM node,
 * including a series of convenience methods based on the {@link
 * DOMUtils} class.  The base class also contains an {@link #equals}
 * method that compares the underlying DOM nodes rather than the
 * wrapper classes.</p>
 *
 * <p>Additionally, since Java does not allow multiple inheritance,
 * this class provides default implementations for all of the methods
 * declared in the common *Node interfaces (except for OriginNode and
 * PrimaryNode).  The interfaces will hide unused methods from the
 * user.</p>
 *
 * <p>Finally, this class provides convenience methods for accessing
 * the underlying DOM tree.</p>
 *
 * @author Reuters PLC
 * @version 1.0
 */
class DOMBaseNode
    implements AssignmentNode, BaseNode, CatalogNode, CommentNode,
	       HrefNode, IdNode, LanguageNode, PropertyNode,
	       RevisionHistory
{

    /**
     * Constructor.
     *
     * @param session The NewsML session for the package.
     */
    protected DOMBaseNode (Node element, DOMNewsMLSession session)
    {
	if (element == null)
	    throw new NullPointerException("null DOM Element node");
	this.element = element;
	this.session = session;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the AssignmentNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see AssignmentNode#getAssignedBy()
     */
    public String getAssignedBy ()
    {
	return getDOMAttribute("AssignedBy");
    }


    /**
     * @see AssignmentNode#getImportance()
     */
    public String getImportance ()
    {
	return getDOMAttribute("Importance");
    }


    /**
     * @see AssignmentNode#getConfidence()
     */
    public String getConfidence ()
    {
	return getDOMAttribute("Confidence");
    }


    /**
     * @see AssignmentNode#getHowPresent()
     */
    public String getHowPresent ()
    {
	return getDOMAttribute("HowPresent");
    }


    /**
     * @see AssignmentNode#getAssignmentDateAndTime()
     */
    public String getAssignmentDateAndTime ()
    {
	return getDOMAttribute("DateAndTime");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the BaseNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see BaseNode#getXMLName
     */
    public String getXMLName ()
    {
	return element.getNodeName();
    }


    /**
     * @see NewsMLSession#getSession
     */
    public NewsMLSession getSession ()
    {
	return session;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the CatalogNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see CatalogNode#getCatalog()
     */
    public Catalog getCatalog ()
    {
	Node e = getDOMChild("Catalog");
	return (e == null ? null : new DOMCatalog(e, session));
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the CommentNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see CommentNode#getCommentCount
     */
    public int getCommentCount ()
    {
	return DOMUtils.count(getDOMNode(), "Comment");
    }


    /**
     * @see CommentNode#getComment(int)
     */
    public Comment getComment (int index)
    {
	Node e = getDOMChild("Comment", index);
	return (e == null ? null : new DOMText(e, session));
    }


    /**
     * @see CommentNode#getComment()
     */
    public Comment [] getComment ()
    {
	int len = getCommentCount();
	Comment ret[] = new Comment[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getComment(i);
	return ret;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the HrefNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see HrefNode#getHref()
     */
    public String getHref ()
    {
	return getDOMAttribute("Href");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the IdNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see IdNode#getDuid()
     */
    public String getDuid ()
    {
	return getDOMAttribute("Duid");
    }


    /**
     * @see IdNode#getEuid()
     */
    public String getEuid ()
    {
	return getDOMAttribute("Euid");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the LanguageNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see LanguageNode#getLang()
     */
    public String getLang ()
    {
	return getDOMAttribute("xml:lang");
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of the PropertyNode interface.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see PropertyNode#getPropertyCount()
     */
    public int getPropertyCount ()
    {
	return getDOMChildCount("Property");
    }


    /**
     * @see PropertyNode#getProperty(int)
     */
    public Property getProperty (int index)
    {
	Node e = getDOMChild("Property", index);
	return (e == null ? null : new DOMProperty(e, session));
    }


    /**
     * @see PropertyNode#getProperty()
     */
    public Property [] getProperty ()
    {
	int len = getPropertyCount();
	Property ret[] = new Property[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getProperty(i);
	return ret;
    }



    ////////////////////////////////////////////////////////////////////
    // DOM convenience methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Get the underlying DOM node for this NewsML object.
     *
     * The underlying node allows users to query information not
     * available directly through this library.
     *
     * @return The underlying DOM node (not null).
     */
    public Node getDOMNode ()
    {
	return element;
    }


    /**
     * Get the qualified name of the current DOM element.
     *
     * @return The name as a string (with prefix, if applicable).
     * @see org.w3c.dom.Node#getNodeName
     */
    public String getDOMName ()
    {
	return getDOMNode().getNodeName();
    }


    /**
     * Get the text content of the current element.
     *
     * Return all of the text in the current element and its descendants.
     *
     * @return The text as a string.
     * @see DOMUtils#getText(org.w3c.dom.Node)
     */
    public String getDOMContent ()
    {
	return DOMUtils.getText(element);
    }


    /**
     * Get the text content of the specified child element.
     *
     * Return all of the text of the first child element with
     * the specified name, including that of its descendants.
     *
     * @param name The name of the child element.
     * @return The child element's text as a string or null
     * if the specified element was not present.
     * @see DOMUtils#getText(org.w3c.dom.Node)
     */
    public String getDOMContent (String name)
    {
	return DOMUtils.getText(DOMUtils.find(element, name));
    }


    /**
     * Get the text content of the specified child element.
     *
     * Return all of the text in the specified child element and its
     * descendants.
     *
     * @param name The name of the child element.
     * @param index The index of the child, zero-based, numbered
     * sequentially in document order.
     * @return The text as a string or null if the specified element
     * was not present.
     * @see DOMUtils#getText(org.w3c.dom.Node)
     */
    public String getDOMContent (String name, int index)
    {
	return DOMUtils.getText(DOMUtils.find(element, name, index));
    }


    /**
     * Get the string value of an attribute.
     *
     * @return The string value of the attribute, or null
     * if it is not present.
     * @see DOMUtils#getAttribute
     */
    public String getDOMAttribute (String name)
    {
	return DOMUtils.getAttribute(element, name);
    }


    /**
     * Get a named child node.
     *
     * Look up the first child with the specified name returning null 
     * if it is not present.
     *
     * @param name The name of the child to look up.
     * @see DOMUtils#find(org.w3c.dom.Node,java.lang.String)
     */
    public Node getDOMChild (String name)
    {
	return DOMUtils.find(element, name);
    }


    /**
     * Return the number of named DOM children.
     *
     * @param name The name of the children to look up.
     */
    public int getDOMChildCount (String name)
    {
	return DOMUtils.count(element, name);
    }


    /**
     * Get a named child node.
     *
     * Look up the child with the specified name and index,
     * returning null if it is not present.
     *
     * @param name The name of the child to look up.
     * @param index The index of the child, zero-based, numbered 
     * sequentially in document order.
     * @see DOMUtils#find(org.w3c.dom.Node,java.lang.String,int)
     */
    public Node getDOMChild (String name, int index)
    {
	return DOMUtils.find(element, name, index);
    }


    /**
     * Test whether two NewsML objects are equal.
     *
     * Two NewsML objects are equal if they share the same subclass
     * and point to the same underlying DOM node.
     *
     * @param o The object to compare.
     * @return true if the objects are equal, false otherwise.
     */
    public boolean equals (Object o)
    {
	boolean retval;

	if (o == null) {
	    retval = false;
	} else if (o == this) {
	    retval = true;
	} else if (o instanceof DOMBaseNode){
	    retval = (getClass().equals(o.getClass()) &&
		      getDOMNode().equals(((DOMBaseNode)o).getDOMNode()));
	} else {
	    retval = false;
	}

	return retval;
    }



    ////////////////////////////////////////////////////////////////////
    // Other useful accessors.
    ////////////////////////////////////////////////////////////////////


    /**
     * Count the formal names for a named child element.
     *
     * @return The number of named children.
     */
    public int getDOMFormalNameCount (String name)
    {
	return getDOMChildCount(name);
    }


    /**
     * Get a formal name built on a child.
     *
     * @return The formal name built on the child, or null if the
     * child is not present.
     */
    public DOMFormalName getDOMFormalName (String name, int index)
    {
	Node e = getDOMChild(name, index);
	return (e == null ? null : new DOMFormalName(e, session));
    }


    /**
     * Get a list of parties.
     *
     * @param name The name of the subelement containing the list.
     * @return A PartyList node, or null if no subelement exists with
     * the specified name.
     */
    public DOMPartyList getDOMPartyList (String name)
    {
	Node e = getDOMChild(name);
	return (e == null ? null : new DOMPartyList(e, session));
    }


    /**
     * Get the number of matching parties.
     *
     * @return The number of parties with the name provided.
     */
    public int getPartyCount (String name)
    {
	return getDOMChildCount(name);
    }


    /**
     * Get a list of parties.
     *
     * @param name The name of the subelement containing the list.
     * @param index The index of the list, zero-based, numbered
     * sequentially.
     * @return A PartyList node, or null if no subelement exists with
     * the specified name and index.
     */
    public DOMPartyList getDOMPartyList (String name, int index)
    {
	Node e = getDOMChild(name, index);
	return (e == null ? null : new DOMPartyList(e, session));
    }


    /**
     * Get text for a specific subelement.
     *
     * @param name The subelement name.
     * @return The subelement's content as a Text node, or null
     * if the subelement was not found.
     */
    public DOMText getDOMText (String name)
    {
	Node e = getDOMChild(name);
	return (e == null ? null : new DOMText(e, session));
    }


    /**
     * Count the number of named text-related elements.
     *
     * @return The number of named text-related elements.
     */
    public int nTexts (String name)
    {
	return getDOMChildCount(name);
    }


    /**
     * Get text for a specific subelement.
     *
     * @param name The subelement name.
     * @param The index of the element among elements with the same
     * name, zero-based, numbered sequentially in document order.
     * @return The subelement's content as a Text node, or null
     * if the subelement was not found.
     */
    public DOMText getDOMText (String name, int index)
    {
	Node e = getDOMChild(name, index);
	return (e == null ? null : new DOMText(e, session));
    }


    /**
     * Get origin text for a specific subelement.
     *
     * @param name The subelement name.
     * @return The subelement's content as an OriginText node, or null
     * if the subelement was not found.
     */
    public DOMOriginBase getDOMOriginBase (String name)
    {
	Node e = getDOMChild(name);
	return (e == null ? null : new DOMOriginBase(e, session));
    }


    /**
     * Get origin text for a specific subelement.
     *
     * @param name The subelement name.
     * @param The index of the element among elements with the same
     * name, zero-based, numbered sequentially in document order.
     * @return The subelement's content as an OriginText node, or null
     * if the subelement was not found.
     */
    public DOMOriginBase getDOMOriginBase (String name, int index)
    {
	Node e = getDOMChild(name, index);
	return (e == null ? null : new DOMOriginBase(e, session));
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private Node element;
    protected DOMNewsMLSession session;	// I'll probably regret making 
				// this visible...

}

// end of DOMBaseNode.java
