// DOMFormalName.java - DOM implementation of AssignedFormalName, FormalName, and OfInterestTo
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC
// $Id: DOMFormalName.java,v 1.12 2001/05/23 01:56:18 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;

import org.w3c.dom.Node;

import org.newsml.toolkit.AssignedFormalName;
import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.Catalog;
import org.newsml.toolkit.DefaultVocabularyFor;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.OfInterestTo;
import org.newsml.toolkit.Resource;
import org.newsml.toolkit.TopicSet;


/**
 * DOM implementation of AssignedFormalName, FormalName, and OfInterestTo
 *
 * @author Reuters PLC
 * @version 1.0
 */
class DOMFormalName
    extends DOMBaseNode
    implements AssignedFormalName, FormalName, OfInterestTo
{

    /**
     * Constructor.
     */
    protected DOMFormalName (Node element, DOMNewsMLSession session)
    {
	super(element, session);
    }


    /**
     * @see FormalName#getName
     */
    public String getName ()
    {
	return getDOMAttribute("FormalName");
    }


    /**
     * @see FormalName#getVocabulary
     */
    public String getVocabulary ()
    {
	return getDOMAttribute("Vocabulary");
    }


    /**
     * @see FormalName#getDefaultVocabulary
     */
    public String getDefaultVocabulary (boolean useExternal)
	throws IOException
    {
	String vocab = getVocabulary();
	if (vocab != null) {
	    return vocab;
	} else {
	    Decl decl = getDefaultDecl(getDOMNode().getParentNode(),
				       getDOMNode(),
				       useExternal);
	    if (decl != null) {
		if (decl.resource.getUrlCount() > 0)
		    return decl.resource.getUrl(0).getValue();
		else
		    return decl.resource.getUrn().getValue();
	    } else {
		return null;
	    }
	}
    }


    /**
     * @see FormalName#getScheme
     */
    public String getScheme ()
    {
	return getDOMAttribute("Scheme");
    }


    /**
     * @see FormalName#getScheme
     */
    public String getDefaultScheme (boolean useExternal)
	throws IOException
    {
	String scheme = getScheme();
	if (scheme != null)
	    return scheme;
	Decl decl = getDefaultDecl(getDOMNode().getParentNode(),
				   getDOMNode(),
				   useExternal);
	if (decl == null)
	    return null;
	else
	    return decl.defaultVocabularyFor.getScheme();
    }


    /**
     * @see FormalName#getTopicSet
     */
    public TopicSet getTopicSet (boolean useExternal)
	throws IOException
    {
	String vocab = getDefaultVocabulary(useExternal);
	if (vocab == null)
	    return null;

	BaseNode node = getSession().findReference(vocab, useExternal);
	if (node == null || !("TopicSet".equals(node.getXMLName()))) {
	    return null;
	} else {
	    return (TopicSet)node;
	}
    }


    /**
     * @see FormalName#toString()
     */
    public String toString ()
    {
	return makeString(getVocabulary(), getScheme(), getName());
    }


    /**
     * @see FormalName#toString(boolean)
     */
    public String toString (boolean useExternal)
	throws IOException
    {
	return makeString(getDefaultVocabulary(useExternal),
			  getDefaultScheme(useExternal),
			  getName());
    }


    /**
     * @see OfInterestTo#getRelevance()
     */
    public AssignedFormalName getRelevance ()
    {
	return getDOMFormalName("Relevance", 0);
    }


    /**
     * Construct a string version of the name parts.
     */
    private String makeString (String vocab, String scheme, String name)
    {
	StringBuffer buf = new StringBuffer();
	buf.append('{');
	buf.append(vocab == null ? "" : vocab);
	buf.append("}{");
	buf.append(scheme == null ? "" : scheme);
	buf.append('}');
	buf.append(name);
	return buf.toString();
    }



    ////////////////////////////////////////////////////////////////////
    // Internal methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Look up a default vocabulary for an element.
     *
     * This method currently supports only element names, not full
     * XPath expressions.  This method climbs recursively to the
     * root element of the document, check each element to see if it
     * has a Catalog child containing an appropriate
     * DefaultVocabularyFor declaration.
     *
     * @param node The first node to search (usually starting with the
     * parent element of the element containing the formal name.
     * @return The Resource element that provides the default
     * vocabulary, or null if none was specified.
     */
    private Decl getDefaultDecl (Node startNode, Node targetNode,
				 boolean useExternal)
	throws IOException
    {
	Node c = DOMUtils.find(startNode, "Catalog");
	if (c != null) {
				// Make a catalog
	    Catalog catalog =
		new DOMCatalog(c, session).getReferencedCatalog(useExternal);
	    if (catalog != null) {

				// Look through every default vocab
				// declaration in every resource
		int i = 0;
		Resource r = catalog.getResource(i);
		while (r != null) {
		    int j = 0;
		    DefaultVocabularyFor dv = r.getDefaultVocabularyFor(j);
		    while (dv != null) {
			DOMXPath path = new DOMXPath(dv.getContext());
			if (path.matches(targetNode))
			    return new Decl(r, dv);
			j++;
			dv = r.getDefaultVocabularyFor(j);
		    }
		    i++;
		    r = catalog.getResource(i);
		}
	    }
	}
				// If we arrive here, we failed, so
				// try the parent.
	Node e = startNode.getParentNode();
	return (e == null ? null : getDefaultDecl(e, targetNode, useExternal));
    }


    /**
     * Inner class for a default declaration.
     */
    class Decl
    {
	Decl (Resource resource, DefaultVocabularyFor defaultVocabularyFor)
	{
	    this.resource = resource;
	    this.defaultVocabularyFor = defaultVocabularyFor;
	}

	Resource resource;
	DefaultVocabularyFor defaultVocabularyFor;
    }

}

// end of DOMFormalName.java
