// DOMNewsComponent.java - DOM implementation of NewsComponent.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMNewsComponent.java,v 1.12 2001/05/23 01:56:18 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import org.w3c.dom.Node;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.newsml.toolkit.AdministrativeMetadata;
import org.newsml.toolkit.BasisForChoice;
import org.newsml.toolkit.DescriptiveMetadata;
import org.newsml.toolkit.FormalName;
import org.newsml.toolkit.Metadata;
import org.newsml.toolkit.NewsComponent;
import org.newsml.toolkit.NewsLines;
import org.newsml.toolkit.PrimaryNode;
import org.newsml.toolkit.RightsMetadata;
import org.newsml.toolkit.TopicSet;


/**
 * DOM implementation of NewsComponent.
 *
 * @author Reuters PLC
 * @version 1.0
 */
class DOMNewsComponent
    extends DOMPrimaryNode
    implements NewsComponent
{

    /**
     * Constructor.
     *
     * @param element The DOM node representing the news component.
     */
    protected DOMNewsComponent (Node element, DOMNewsMLSession session)
    {
	super(element, session);
    }


    /**
     * @see NewsComponent#getType()
     */
    public int getType ()
    {
	return PrimaryNode.NEWS_COMPONENT;
    }


    /**
     * @see NewsComponent#isEssential()
     */
    public boolean isEssential ()
    {
	return ("yes".equals(getDOMAttribute("Essential")));
    }


    /**
     * @see NewsComponent#isEquivalentsList()
     */
    public boolean isEquivalentsList ()
    {
	return ("yes".equals(getDOMAttribute("EquivalentsList")));
    }


    /**
     * @see NewsComponent#getTopicSetCount()
     */
    public int getTopicSetCount ()
    {
	return getDOMChildCount("TopicSet");
    }

    
    /**
     * @see NewsComponent#getTopicSet(int)
     */
    public TopicSet getTopicSet (int index)
    {
	Node e = getDOMChild("TopicSet", index);
	return (e == null ? null : new DOMTopicSet(e, session));
    }


    /**
     * @see NewsComponent#getTopicSet()
     */
    public TopicSet [] getTopicSet ()
    {
	int len = getTopicSetCount();
	TopicSet ret[] = new TopicSet[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getTopicSet(i);
	return ret;
    }


    /**
     * @see NewsComponent#getRole()
     */
    public FormalName getRole ()
    {
	return getDOMFormalName("Role", 0);
    }


    /**
     * @see NewsComponent#getBasisForChoiceCount()
     */
    public int getBasisForChoiceCount ()
    {
	return getDOMChildCount("BasisForChoice");
    }


    /**
     * @see NewsComponent#getBasisForChoice(int)
     */
    public BasisForChoice getBasisForChoice (int index)
    {
	Node e = getDOMChild("BasisForChoice", index);
	return (e == null ? null : new DOMBasisForChoice(e, session));
    }


    /**
     * @see NewsComponent#getBasisForChoice()
     */
    public BasisForChoice [] getBasisForChoice ()
    {
	int len = getBasisForChoiceCount();
	BasisForChoice ret[] = new BasisForChoice[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getBasisForChoice(i);
	return ret;
    }


    /**
     * @see NewsComponent#getNewsLines()
     */
    public NewsLines getNewsLines ()
    {
	Node e = getDOMChild("NewsLines");
	return (e == null ? null : new DOMNewsLines(e, session));
    }


    /**
     * @see NewsComponent#getAdministrativeMetadata()
     */
    public AdministrativeMetadata getAdministrativeMetadata ()
    {
	Node e = getDOMChild("AdministrativeMetadata");
	return (e == null ? null
		: new DOMAdministrativeMetadata(e, session));
    }


    /**
     * @see NewsComponent#getRightsMetadata()
     */
    public RightsMetadata getRightsMetadata ()
    {
	Node e = getDOMChild("RightsMetadata");
	return (e == null ? null : new DOMRightsMetadata(e, session));
    }


    /**
     * @see NewsComponent#getDescriptiveMetadata
     */
    public DescriptiveMetadata getDescriptiveMetadata ()
    {
	Node e = getDOMChild("DescriptiveMetadata");
	return (e == null ? null : new DOMDescriptiveMetadata(e, session));
    }


    /**
     * @see NewsComponent#getMetadataCount()
     */
    public int getMetadataCount ()
    {
	return getDOMChildCount("Metadata");
    }


    /**
     * @see NewsComponent#getMetadata(int)
     */
    public Metadata getMetadata (int index)
    {
	Node e = getDOMChild("Metadata", index);
	return (e == null ? null : new DOMMetadata(e, session));
    }


    /**
     * @see NewsComponent#getMetadata()
     */
    public Metadata [] getMetadata ()
    {
	int len = getMetadataCount();
	Metadata ret[] = new Metadata[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getMetadata(i);
	return ret;
    }


    /**
     * @see NewsComponent#getMemberCount()
     */
    public int getMemberCount ()
    {
	int n = 0;
	NodeList children = getDOMNode().getChildNodes();
	int nChildren = children.getLength();
	for (int i = 0; i < nChildren; i++) {
	    Node item = children.item(i);
	    if (item.getNodeType() != Node.ELEMENT_NODE) {
		continue;
	    } else if ("NewsItem".equals(item.getNodeName())) {
		n++;
	    } else if ("NewsComponent".equals(item.getNodeName())) {
		n++;
	    } else if ("NewsItemRef".equals(item.getNodeName())) {
		n++;
	    } else if ("ContentItem".equals(item.getNodeName())) {
		n++;
	    }
	}
	return n;
    }


    /**
     * @see NewsComponent#getMember(int)
     */
    public PrimaryNode getMember (int index)
    {
	NodeList children = getDOMNode().getChildNodes();
	int nChildren = children.getLength();
	for (int i = 0; i < nChildren; i++) {
	    Node item = children.item(i);
	    if (item.getNodeType() != Node.ELEMENT_NODE) {
		continue;
	    } else if ("NewsItem".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMNewsItem(item, session);
		else
		    index--;
	    } else if ("NewsComponent".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMNewsComponent(item, session);
		else
		    index--;
	    } else if ("NewsItemRef".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMNewsItemRef(item, session);
		else
		    index--;
	    } else if ("ContentItem".equals(item.getNodeName())) {
		if (index == 0)
		    return new DOMContentItem(item, session);
		else
		    index--;
	    }
	}
	return null;
    }


    /**
     * @see NewsComponent#getMember()
     */
    public PrimaryNode [] getMember ()
    {
	int len = getMemberCount();
	PrimaryNode ret[] = new PrimaryNode[len];
	for (int i = 0; i < len; i++)
	    ret[i] = getMember(i);
	return ret;
    }

}

// end of DOMNewsComponent.java
