// DOMNewsMLSession.java - DOM implementation of NewsMLSession.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMNewsMLSession.java,v 1.6 2001/05/23 01:56:19 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;

import org.w3c.dom.Node;

import org.newsml.toolkit.BaseNode;
import org.newsml.toolkit.NewsML;
import org.newsml.toolkit.NewsMLException;
import org.newsml.toolkit.NewsMLFactory;
import org.newsml.toolkit.NewsMLSession;


/**
 * DOM implementation of NewsMLSession.
 *
 * @author Reuters PLC
 * @version 1.0
 * @see NewsMLSession
 */
class DOMNewsMLSession
    implements NewsMLSession
{


    ////////////////////////////////////////////////////////////////////
    // Constructor.
    ////////////////////////////////////////////////////////////////////

    /**
     * Constructor.
     *
     * @param root The root node of the NewsML package.
     * @param factory The factory for creating new NewsML packages.
     */
    protected DOMNewsMLSession (DOMNewsMLFactory factory)
    {
	this.factory = factory;
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of NewsMLSession.
    ////////////////////////////////////////////////////////////////////


    /**
     * @see NewsMLSession#getFactory()
     */
    public NewsMLFactory getFactory ()
    {
	return factory;
    }


    /**
     * @see NewsMLSession#getRootNode()
     */
    public BaseNode getRootNode ()
    {
	return root;
    }


    /**
     * @see NewsMLSession#findReference(String,boolean)
     */
    public BaseNode findReference (String reference, boolean useExternal)
	throws IOException
    {
				// Internal reference
	if (reference.startsWith("#")) {
	    String duid = reference.substring(1);
	    Node rootNode = root.getDOMNode();
	    Node targetNode =
		DOMUtils.findElementByAttribute(rootNode, "Duid", duid);
	    return makeNewsMLNode(targetNode);
	}

				// External reference (allowed)
	else if (useExternal) {
	    String url;
	    String duid;
	    int hash = reference.indexOf('#');
	    if (hash == -1) {
		url = reference;
		duid = null;
	    } else {
		url = reference.substring(0, hash);
		duid = reference.substring(hash);
	    }
	    BaseNode node = getFactory().createNode(resolveURL(url));
	    if (duid == null)
		return node;
	    else
		return node.getSession().findReference(duid, false);
	}

				// External (not allowed)
	else {
				// FIXME: should we throw an exception
				// here?
	    return null;
	}
    }



    /**
     * @see NewsMLSession#getBaseURL()
     */
    public String getBaseURL ()
    {
	return baseURL;
    }


    /**
     * @see NewsMLSession#resolveHref(String)
     */
    public URL resolveHref (String href)
	throws MalformedURLException
    {
	return new URL(resolveURL(href));
    }



    ////////////////////////////////////////////////////////////////////
    // Protected methods.
    ////////////////////////////////////////////////////////////////////


    /**
     * Set the root node.
     *
     * @param root The new root node.
     * @see #getRootNode
     */
    protected void setRootNode (DOMBaseNode root)
    {
	this.root = root;
    }


    /**
     * Set the base URL.
     *
     * @param baseURL The base URL for the session.
     * @see #getBaseURL
     */
    protected void setBaseURL (String baseURL)
    {
	this.baseURL = baseURL;
    }


    /**
     * Construct a NewsML node from a DOM node.
     *
     * @param element The DOM element (or null).
     * @return A NewsML node of some kind, or null if the DOM node
     * was null.
     */
    protected DOMBaseNode makeNewsMLNode (Node element)
	throws IOException
    {
	if (element == null)
	    return null;

	String name = element.getNodeName();

				// FIXME: severely restricted
				// for now.

	if (name.equals("NewsML"))
	    return new DOMNewsML(element, this);

	else if (name.equals("NewsItem"))
	    return new DOMNewsItem(element, this);
	
	else if (name.equals("TopicSet"))
	    return new DOMTopicSet(element, this);
	
	else if (name.equals("Topic"))
	    return new DOMTopic(element, this);
	
	else if (name.equals("Catalog"))
	    return new DOMCatalog(element, this);
	
	else
	    throw new NewsMLException("Unsupported root element " + name);
    }


    /**
     * Resolve a URL against the current relative URL.
     */
    protected String resolveURL (String url)
    {
	if (url.startsWith("urn:"))
	    return url;

	try {
	    return new URL(new URL(baseURL), url).toString();
	} catch (MalformedURLException ex) {
	    return url;
	}
    }



    ////////////////////////////////////////////////////////////////////
    // Internal state.
    ////////////////////////////////////////////////////////////////////

    private DOMBaseNode root;
    private DOMNewsMLFactory factory;
    private String baseURL;

}

// end of DOMNewsMLSession.java
