// DOMUtils.java - static utility methods for querying a DOM.
// Copyright (c) 2000 by Reuters PLC and Wavo Company, Inc.
// Copyright (c) 2001 by Reuters PLC.
// $Id: DOMUtils.java,v 1.9 2001/05/23 01:56:19 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;


/**
 * Static utility methods for querying a DOM tree.
 *
 * @author Reuters PLC
 * @version 1.0
 */
final class DOMUtils
{

    /**
     * Private constructor.
     */
    private DOMUtils ()
    {
	// This is never invoked.
    }


    /**
     * Count the child elements with the specified name.
     */
    static int count (Node node, String name)
    {
	return count(node.getChildNodes(), name);
    }


    /**
     * Count the members of a node list with the specified name.
     */
    static int count (NodeList nodes, String name)
    {
	int count = 0;
	int len = nodes.getLength();
	for (int i = 0; i < len; i++) {
	    if (name.equals(nodes.item(i).getNodeName()))
		count++;
	}
	return count;
    }


    /**
     * Find the first child element with the specified name.
     *
     * @param e The parent element, or null (always fails).
     * @param name The name of the child.
     * @return The DOM node for the child element, or null if
     * none was found or a null element was provided.
     * @see #find(org.w3c.dom.Node,java.lang.String name,int index)
     */
    static Node find (Node e, String name)
    {
	if (e == null)
	    return null;
	else
	    return find(e.getChildNodes(), name, 0);
    }


    /**
     * Find the nth child element with the specified name.
     *
     * @param e The parent element, or null (always fails).
     * @param name The name of the child.
     * @param index The index of the child.
     * @return The DOM node for the child element, or null if
     * none was found or a null element was provided.
     * @see #find(org.w3c.dom.Node,java.lang.String name)
     */
    static Node find (Node e, String name, int index)
    {
	if (e == null)
	    return null;
	else
	    return find(e.getChildNodes(), name, index);
    }


    /**
     * Find an element in a DOM node list.
     *
     * @param nodes The node list to search, or null (always fails).
     * @param name The target element name.
     * @param index The target index.
     * @return The DOM node for the matching member of the node list,
     * or null if none was found or a null nodelist was provided.
     */
    static Node find (NodeList nodes, String name, int index)
    {
	int len = nodes.getLength();

	for (int i = 0; i < len; i++) {
	    Node node = nodes.item(i);
	    if (node.getNodeName().equals(name)) {
		index--;
		if (index < 0) {
		    return node;
		}
	    }
	}
	return null;
    }



    /**
     * Find the position of a named, indexed member of a node list.
     *
     * @param nodes The node list to search.
     * @param name The node name.
     * @param index The node index (among nodes with the same name).
     * @return The member's position, or -1 if none was found with
     * the specified name and index.
     */
    static int findPos (NodeList nodes, String name, int index)
    {
	int len = nodes.getLength();
	for (int i = 0; i < len; i++) {
	    Node node = nodes.item(i);
	    if (name.equals(node.getNodeName())) {
		index--;
		if (index < 0)
		    return i;
	    }
	}
	return -1;
    }


    /**
     * Get an attribute value.
     *
     * @param e The element node, or null (always fails).
     * @param name The attribute name.
     * @return The text of the attribute value, or null if the
     * element was null or the attribute was not present.
     */
    static String getAttribute (Node e, String name)
    {
	if (e == null)
	    return null;
	NamedNodeMap atts = e.getAttributes();
	Node att = atts.getNamedItem(name);
	if (att != null)
	    return att.getNodeValue();
	else
	    return null;
    }


    /**
     * Get the text of an element.
     *
     * @param node The element node, or null (always fails).
     * @return A string containing the text of the element and
     * its descendants, or null if the element is null.
     */
    static String getText (Node node)
    {
	if (node == null)
	    return null;
	else
	    return getText(node.getChildNodes());
    }


    /**
     * Get the text of a node list.
     *
     * @param nodes The node list.
     * @return All of the text descendants of the node list in a
     * single string.
     */
    static String getText (NodeList nodes)
    {
	if (nodes == null)
	    return null;
	int len = nodes.getLength();
	StringBuffer text = new StringBuffer();
	for (int i = 0; i < len; i++) {
	    Node child = nodes.item(i);
	    if (child.getNodeType() == Node.TEXT_NODE)
		text.append(child.getNodeValue());
	    else if (child.getNodeType() == Node.ELEMENT_NODE)
		text.append(getText(child));
	}
	return text.toString();
    }


    /**
     * Find a DOM node with the specified attribute and value.
     *
     * This method currently uses a left-right recursive descent
     * search.
     *
     * @param node The starting node in the DOM tree.
     * @param name The attribute name.
     * @param value The attribute value.
     * @return The node of the first element with the specified
     * attribute and value, or null if none was found.
     */
    static Node findElementByAttribute (Node node, String name, String value)
    {
				// If this node matches, return it.
	if (node.getNodeType() == Node.ELEMENT_NODE &&
	    value.equals(((Element)node).getAttribute(name))) {
	    return node;
	}

				// If there are children, walk through them
	else if (node.hasChildNodes()) {
	    NodeList children = node.getChildNodes();
	    int nChildren = children.getLength();
	    for (int i = 0; i < nChildren; i++) {
		Node retval =
		    findElementByAttribute(children.item(i), name, value);
		if (retval != null)
		    return retval;
	    }
	}

				// OK, not found.
	return null;
    }

}

// end of DOMUtils.java
