// DOMXPath.java - a compiled XPath expression for DOM node matching.
// Copyright (c) 2001 by Reuters PLC
// $Id: DOMXPath.java,v 1.7 2001/05/23 01:56:19 david Exp $

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.newsml.toolkit.dom;

import java.util.Vector;

import org.w3c.dom.Node;

import org.newsml.toolkit.NewsMLException;


/**
 * A compiled XPath expression for DOM node matching.
 *
 * <p>This class currently supports only trivially simple XPath
 * expressions, but it can grow over time into a full XPath library.
 * The class does not depend on anything else in this package, so some
 * day it can spin off into a separate library.</p>
 *
 * @author Reuters PLC
 * @version 1.0
 */
class DOMXPath
{
  
    /**
     * Constructor.
     *
     * <p>This constructor compiles its argument (which must be an
     * XPath expression) for future matching.  It does not currently
     * throw an exception for an invalid XPath string, but it will do
     * so in the future.</p>
     *
     * @param path An XPath expression as a string.
     * @exception NewsMLException if the XPath expression is invalid.
     */
    DOMXPath (String path)
	throws NewsMLException
    {
	compile(path);
    }


    /**
     * Test whether a DOM node matches an XPath expression.
     *
     * <p>This method tests whether an the current XPath expression
     * (as supplied to the constructor) matches the current DOM node.
     * The current node is assumed to appear at the end of the
     * path.</p>
     *
     * <p>Currently, this method supports only trivially simple XPath
     * expressions; in particular, it matches only a series of simple
     * element names separated by slashes, and does not recognize a
     * leading slash as the document root.</p>
     *
     * @param node The DOM node to match.
     * @return true if the node matches the expression, false
     * otherwise.
     */    
    public boolean matches (Node node)
    {
	return matches(node, components.size() - 1);
    }


    /**
     * Tail-recursive method for matching a full path.
     *
     * <p>Match each component of the path in sequence, starting
     * with the target node.</p>
     *
     * @param node The node to test.
     * @param index The index in the components list.
     * @return true if this component and all higher ones match
     * this node and its ancestors; false otherwise.
     */
    private boolean matches (Node node, int index)
    {
	if (index < 0)
	    return true;
	else if (node == null)
	    return false;

	Component component = (Component)components.elementAt(index);
	if (!component.matches(node))
	    return false;
	else
	    return matches(node.getParentNode(), index - 1);
    }


    /**
     * Compile an XPath into its internal representation.
     */
    private void compile (String path)
	throws NewsMLException
    {
	components = new Vector();
	String part;
	int pos = 0;
	int index = 0;
	index = path.indexOf('/', pos);
	while (index != -1) {
	    part = path.substring(pos, index);
	    if (part.length() > 0)
		components.addElement(new Component(part));
	    else
		badPath("Empty XPath component", path);
	    pos = index + 1;
	    index = path.indexOf('/', pos);
	}
	part = path.substring(pos);
	if (part.length() > 0)
	    components.addElement(new Component(part));
	else
	    badPath("Empty XPath component", path);
    }


    private void badPath (String message, String path)
	throws NewsMLException
    {
	throw new NewsMLException("Unsupported XPath: " +
				  message +
				  " in \"" +
				  path +
				  '"');
    }

    private Vector components;


    /**
     * A compiled component of an XPath.
     */
    class Component
    {

	/**
	 * Constructor.
	 */
	Component (String path)
	    throws NewsMLException
	{
	    compile(path);
	}


	/**
	 * Test whether an XPath component matches a single node.
	 */
	boolean matches (Node node)
	{
				// The element name always has to match
	    if (!elementName.equals(node.getNodeName()))
		return false;

				// Match the attribute name if one
				// was provided.
	    if (attributeName != null) {
		Node att = node.getAttributes().getNamedItem(attributeName);
		if (att == null || !attributeValue.equals(att.getNodeValue()))
		    return false;
	    }

	    return true;
	}


	/**
	 * Compile an XPath component.
	 */
	private void compile (String path)
	    throws NewsMLException
	{
	    int index = path.indexOf('[');
	    if (index == -1) {
		elementName = path;
	    } else {
		elementName = path.substring(0, index);
		compileTests(path.substring(index+1));
	    }
	}

	/**
	 * Compile an XPath component's tests.
	 *
	 * <p>This is a quick-and-dirty kludge right now.</p>
	 */
	private void compileTests (String tests)
	    throws NewsMLException
	{
	    if (tests.charAt(0) != '@')
		badPath("Only attribute tests supported (expected '@')",
			tests);
	    int index = tests.indexOf('=');
	    if (index == -1)
		badPath("Only '=' supported in node tests", tests);
	    attributeName = tests.substring(1, index);
	    index++;
	    char delim = tests.charAt(index);
	    if (delim != '"' && delim != '\'')
		badPath("Expected ' or \"", tests);
	    index ++;
	    int index2 = tests.indexOf(delim, index);
	    if (index2 == -1)
		badPath("Unterminated literal", tests);
	    attributeValue = tests.substring(index, index2);
	}

	private String elementName;
	private String attributeName;
	private String attributeValue;
    }

}

// end of DOMXPath.java

